"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

// PLACEHOLDER-ADEIRA-UNIVERSE-INTERNAL


var template = require('@babel/template').default;

var t = require('@babel/types');

var murmurHash = require('@adeira/murmur-hash').default;

var childProcess = require('child_process');

var findCacheDir = require('find-cache-dir');

var fs = require('fs');

var path = require('path');

var hash = require('object-hash');

var getCssDeclarations = require('./getCssDeclarations').default;

var TemplateLiteralHandler = require('./TemplateLiteralHandler').default;

var PLUGIN_NAME = 'sx-tailwind-babel-transform';

module.exports = function sxTailwindBabelPlugin() {
  var stylesCollector = [];
  var tailwindCallee = '';
  var stylesVarName = 'styles';
  var styles = {};
  var keyframes = {};
  return {
    name: PLUGIN_NAME,
    pre: function pre(state) {
      var cacheDir = findCacheDir({
        name: PLUGIN_NAME
      });

      if (!fs.existsSync(cacheDir)) {
        fs.mkdirSync(cacheDir, {
          recursive: true
        });
      }

      var options = state.opts.plugins.find(function (p) {
        return p.key === PLUGIN_NAME;
      }).options;
      var cacheFile = path.join(cacheDir, "styles_".concat(hash(options), ".json"));

      if (!fs.existsSync(cacheFile)) {
        // This way async code can be called in sync process (Babel is sync, Tailwind processor async)
        childProcess.execFileSync('node', ["".concat(getCurrentDir(), "createCache.js"), JSON.stringify({
          options: options,
          cacheFile: cacheFile
        })]);
      }

      var cache = require(cacheFile);

      keyframes = cache.keyframes;
      styles = cache.styles;
    },
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@adeira/sx-tailwind') {
          path.node.specifiers.forEach(function (_ref) {
            var imported = _ref.imported,
                local = _ref.local;

            if (imported.name === 'tailwind') {
              tailwindCallee = local.name;
            }
          });
          var sxImport = template.ast("import sx from '@adeira/sx'");
          path.replaceWith(sxImport);
        }
      },
      CallExpression: function CallExpression(path) {
        if (path.node.callee.name === tailwindCallee) {
          path.traverse({
            TemplateLiteral: function TemplateLiteral(path) {
              TemplateLiteralHandler(path, stylesCollector, stylesVarName);
            }
          });

          if (path.node.arguments.length === 1 && path.node.arguments[0].type === 'StringLiteral') {
            var _stylesCollector;

            var _styles = path.node.arguments[0].value.split(' ').filter(function (s) {
              return s !== '';
            });

            (_stylesCollector = stylesCollector).push.apply(_stylesCollector, (0, _toConsumableArray2.default)(_styles));

            path.replaceWith(t.callExpression(t.identifier(stylesVarName), _styles.map(function (style) {
              return t.stringLiteral(style);
            })));
          }
        }
      },
      Program: {
        enter: function enter(path, state) {
          stylesCollector = [];
          var filename = state.file.opts.filename.split('/').slice(-2).join('/');
          stylesVarName = "__styles_".concat(murmurHash(filename));
        },
        exit: function exit(path) {
          var declarations = Object.fromEntries(stylesCollector.map(function (style) {
            return [style, getCssDeclarations(style, keyframes, styles)];
          }));

          if (Object.keys(declarations).length > 0) {
            path.node.body.push(template.ast("const ".concat(stylesVarName, " = sx.create(").concat(JSON.stringify(declarations), ")"))); // if animation is used, transform "sx.keyframe" string into call expression

            path.traverse({
              ObjectProperty: function ObjectProperty(path) {
                if (typeof path.node.key.value !== 'string' || !path.node.key.value.startsWith('--animation-name-')) {
                  return;
                }

                var sxKeyframe = template.ast(path.node.value.value);
                path.node.value = sxKeyframe.expression;
              }
            });
          } // is there SX imported twice?


          var sxImports = path.node.body.reduce(function (a, _ref2, index) {
            var type = _ref2.type,
                source = _ref2.source,
                specifiers = _ref2.specifiers;

            if (type === 'ImportDeclaration' && source.value === '@adeira/sx' && Array.isArray(specifiers) && specifiers[0].type === 'ImportDefaultSpecifier') {
              a.push(index);
            }

            return a;
          }, []);

          if (sxImports.length > 1) {
            delete path.node.body[sxImports[0]];
          }
        }
      }
    }
  };
};

function getCurrentDir() {
  var monorepoDir = './src/babel-plugin-transform-sx-tailwind/src/';
  var packageDir = './node_modules/@adeira/babel-plugin-transform-sx-tailwind/src/';
  var vercelDir = '../babel-plugin-transform-sx-tailwind/src/';

  if (fs.existsSync(monorepoDir)) {
    return monorepoDir;
  }

  if (fs.existsSync(packageDir)) {
    return packageDir;
  }

  if (fs.existsSync(vercelDir)) {
    return vercelDir;
  }

  throw Error("Error, can't detect current working dir! cwd: ".concat(process.cwd()));
}